import * as Immutable from "immutable";
export type AllowedValue = string | number | boolean | IAllowedMap | IAllowedList | symbol | IImmutalbeMap<AnyObj> | AnyObj;
type IAllowedMap = Immutable.Map<string, AllowedValue>;
type IAllowedList = Immutable.List<AllowedValue>;
export type MapTypeAllowedData<DataType> = {
    [K in keyof DataType]: DataType[K];
};
export interface IImmutalbeMap<DataType extends MapTypeAllowedData<DataType>, origin extends AnyObj = DataType> extends Immutable.Map<keyof DataType, AllowedValue> {
    toJS(): origin;
    get<K extends keyof DataType>(key: K, notSetValue?: DataType[K]): DataType[K];
    set<K extends keyof DataType>(key: K, value: DataType[K]): this;
    update<K extends keyof DataType>(key: K, updater: (value: DataType[K]) => DataType[K]): this;
    update<K extends keyof DataType>(key: K, notSetValue: DataType[K], updater: (value: DataType[K]) => DataType[K]): this;
    update<R>(updater: (value: this) => R): R;
    updateIn<R = any>(path: string | string[], updater: (value: R) => R): this;
}
export interface IImmutalbeList<T> extends Immutable.List<T> {
    toJS<orgin>(): orgin[];
    updateIn<R = any>(path: (number | string)[], updater: (value: R) => R): this;
    getIn<R = any>(path: (number | string)[]): R;
    push(...values: Array<T>): this;
    clear(): this;
    set(index: number, v: T): this;
    delete(index: number): this;
}
export type ImmuTreeNode<T extends AnyObj, K extends string = "children"> = Omit<T, K> & {
    [f in K]: IImmutalbeList<IImmutalbeMap<ImmuTreeNode<T, f>, T>>;
};
export type immuObjArr<T extends AnyObj, K extends keyof T> = Omit<T, K> & {
    [f in K]: T[f] extends AnyObj[] ? IImmutalbeList<IImmutalbeMap<T[f][0]>> : IImmutalbeMap<T[f]>;
};
declare module "Immutable" {
    function fromJS<T>(data: any): T;
}
export {};
